/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.department.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.dev.modular.file.service.DevFileService;
import vip.xiaonuo.modular.department.entity.Department;
import vip.xiaonuo.modular.department.mapper.DepartmentMapper;
import vip.xiaonuo.modular.department.param.DepartmentAddParam;
import vip.xiaonuo.modular.department.param.DepartmentEditParam;
import vip.xiaonuo.modular.department.param.DepartmentIdParam;
import vip.xiaonuo.modular.department.param.DepartmentPageParam;
import vip.xiaonuo.modular.department.service.DepartmentService;

import java.util.List;

/**
 * 部门Service接口实现类
 *
 * @author newspaper
 * @date  2023/12/13 08:44
 **/
@Service
public class DepartmentServiceImpl extends ServiceImpl<DepartmentMapper, Department> implements DepartmentService {

    private static final String SNOWY_SYS_DEFAULT_FILE_ENGINE_KEY = "SNOWY_SYS_DEFAULT_FILE_ENGINE";

    @Autowired
    private DevFileService devFileService;

    @Override
    public Page<Department> page(DepartmentPageParam departmentPageParam) {
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(departmentPageParam.getSortField(), departmentPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(departmentPageParam.getSortOrder());
            queryWrapper.orderBy(true, departmentPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(departmentPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(Department::getSortCode);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(DepartmentAddParam departmentAddParam, MultipartFile departmentImgUrl) {
        Department department = BeanUtil.toBean(departmentAddParam, Department.class);
        if (ObjectUtil.isNotEmpty(departmentImgUrl)){
            String uploadReturnUrl = devFileService.uploadReturnUrl(SNOWY_SYS_DEFAULT_FILE_ENGINE_KEY, departmentImgUrl);
            department.setDepartmentImgUrl(uploadReturnUrl);
        }
        this.save(department);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(DepartmentEditParam departmentEditParam, MultipartFile departmentImgUrl) {
        Department department = this.queryEntity(departmentEditParam.getId());
        BeanUtil.copyProperties(departmentEditParam, department);
        if (ObjectUtil.isNotEmpty(departmentImgUrl)){
            String uploadReturnUrl = devFileService.uploadReturnUrl(SNOWY_SYS_DEFAULT_FILE_ENGINE_KEY, departmentImgUrl);
            department.setDepartmentImgUrl(uploadReturnUrl);
        }
        this.updateById(department);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<DepartmentIdParam> departmentIdParamList) {
        for (DepartmentIdParam departmentIdParam : departmentIdParamList) {
            if (this.count(new QueryWrapper<Department>().eq("parent_id",departmentIdParam.getId())) > 0) {
                throw new CommonException("部门存在下级,请先删除下级部门!");
            }
        }
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(departmentIdParamList, DepartmentIdParam::getId));
    }

    @Override
    public Department detail(DepartmentIdParam departmentIdParam) {
        return this.queryEntity(departmentIdParam.getId());
    }

    @Override
    public Department queryEntity(String id) {
        Department department = this.getById(id);
        if(ObjectUtil.isEmpty(department)) {
            throw new CommonException("部门不存在，id值为：{}", id);
        }
        return department;
    }

    @Override
    public List<Department> departmentSelector() {
        String orgId = StpLoginUserUtil.getLoginUser().getOrgId();
        List<Department> departmentList = this.list(new QueryWrapper<Department>().lambda().eq(Department::getOrgId, orgId));
        return departmentList;
    }
}
